module uart_ctrl(
	clk,
	rst_n,
	txd_rxd,
	write,
	rx_sig,
	read,
	din,
	dout,
	en);

	//---Ports declearation: generated by Robei---
	input clk;
	input rst_n;
	input txd_rxd;
	output write;
	output rx_sig;
	input read;
	input [7:0] din;
	output [7:0] dout;
	input en;

	wire clk;
	wire rst_n;
	wire txd_rxd;
	reg write;
	reg rx_sig;
	wire read;
	wire [7:0] din;
	reg [7:0] dout;
	wire en;

	//----Code starts here: integrated by Robei-----
	parameter  F_SYS_CLK  =  33_333_333; //system clock frequency
	parameter  BAUD_RATE  =  115200; //baud rate
	
	parameter  IDLE=3'd0,
	                  START=3'd1,
	                  SEND_DATA=3'd2,
	                  RECIVE_DATA=3'd3,
	                  STOP=3'd4; //state define
	          
	parameter  CNT_BAUD  =  F_SYS_CLK/BAUD_RATE;
	parameter  CNT_CLK  =  F_SYS_CLK/BAUD_RATE/2;
	    
	reg [7:0] data_reg;
	reg [3:0] cnt_data;
	reg [24:0] cnt_clk;
	reg baud_clk;
	reg [2:0] state;
	reg [2:0] next_state;
	reg clk_en; 
	
	    
	//baud clock enable
	always@(posedge clk or negedge rst_n)begin
			if(~rst_n)
	     			clk_en<=1'b0;
			else if(state==START)
					clk_en<=1'b1;
	     else if(state==IDLE)
	           clk_en<=1'b0; //do not making baud clock before start state
	     else
	           clk_en<=clk_en;
	end
	    
	//cnt_clk
	always@ (posedge clk or negedge rst_n)begin
			if(~rst_n)
					cnt_clk<=25'b0;
	     else if(clk_en)
					if(cnt_clk==CNT_BAUD-1)
	                cnt_clk<=25'b0;
					else
	                cnt_clk<=cnt_clk+1;
	     else
					cnt_clk<=25'b0;
	end
	  
	//making baud clock
	always@ (posedge clk or negedge rst_n)begin
			if(~rst_n)
	 				baud_clk<=1'b0;
			else if(cnt_clk==CNT_CLK)
					baud_clk<=~baud_clk;
			else
					baud_clk<=baud_clk;
	end
	  
	//the first state machine, describe the transition of the present and next states
	always @(posedge clk or negedge rst_n)begin
			if(!rst_n)
					state<=3'd0;
			else
					state<=next_state;
	end 
	  
	//the second state machine, describes the conditions for state jumps
	always@(*)begin
			if(~rst_n)
					next_state=3'd0;
			else begin 
					case(state)
					IDLE:begin
							if(en) 
									next_state = START;
							else
									next_state = IDLE;
					end
	
					START:begin
							if(cnt_clk==CNT_BAUD-2)begin
									if(txd_rxd==1'b1)
											next_state= SEND_DATA;
									else if(txd_rxd==1'b0 && !read)
											next_state = RECIVE_DATA;
							end
							else
									next_state = START;
					end
	
					SEND_DATA:begin
							if(cnt_data==4'd8)
									next_state = STOP;
							else
									next_state = SEND_DATA;
					end
	
					RECIVE_DATA:begin
							if(cnt_data==4'd8)begin
									next_state = STOP;
									rx_sig<=1'b1;
							end
							else
									next_state = RECIVE_DATA;
					end
	
					STOP:begin
							if(cnt_clk==CNT_BAUD-2)
									next_state = IDLE;
							else
									next_state = STOP;
					end
	
					default: ;
					endcase   
			end                                                                        
	end 
	  
	//the third state machine, describes input and output
	always @(posedge clk or negedge rst_n)begin
			if(!rst_n)begin
					write<=1'b1;
					data_reg<=8'b0000_0000; 
					cnt_data<=4'b0000; 
			end      
			else begin
					case(state)
					IDLE:begin
							write<=1'b1;
							data_reg<=8'b0000_0000;
							cnt_data<=4'b0000; 
					end
	
					START:begin
							write<=1'b0;					
					end
	
					SEND_DATA:begin
							if(cnt_data<4'd8)
									write<=din[cnt_data];
							else
									write<=write;
							if(cnt_clk==CNT_BAUD-2)
									cnt_data<=cnt_data+1;
							else
									cnt_data<=cnt_data;                  
					end
	
					RECIVE_DATA:begin
							if(cnt_data<4'd8)
									data_reg[cnt_data]<=read;
							else
									data_reg[cnt_data]<=data_reg[cnt_data];
							if(cnt_clk==CNT_BAUD-2)
									cnt_data<=cnt_data+1'b1;
							else
									cnt_data<=cnt_data;        
					end
	
					STOP:begin
							cnt_data<=4'b0000;
							if(txd_rxd==1)
									write<=1'b1; //txd stop bit
							if(txd_rxd==0)
									dout<=data_reg;					
					end
	
					default: ;
					endcase
			end      
	end
	
	
endmodule    //uart_ctrl

